/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.runtime.mapper.rdb.adapter.lib;

import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.runtime.mapper.rdb.adapter.api.RdbAdpater;
import org.objectweb.jorm.type.api.PType;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Date;
import java.math.BigDecimal;

/*
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */

public class BasicRdbAdapter implements RdbAdpater {
	/**
	 * This methods return the SQL type code linked to the PType
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	public int getSqlTypeCode(PType pt) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql Type code associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_OBJBOOLEAN:
			return Types.BIT;
		case PType.TYPECODE_CHAR:
		case PType.TYPECODE_OBJCHAR:
			return Types.CHAR;
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_OBJBYTE:
			return Types.TINYINT;
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_OBJSHORT:
			return Types.SMALLINT;
		case PType.TYPECODE_INT:
		case PType.TYPECODE_OBJINT:
			return Types.INTEGER;
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_OBJLONG:
			return Types.BIGINT;
		case PType.TYPECODE_FLOAT:
		case PType.TYPECODE_OBJFLOAT:
			return Types.REAL;
		case PType.TYPECODE_DOUBLE:
		case PType.TYPECODE_OBJDOUBLE:
			return Types.DOUBLE;
		case PType.TYPECODE_STRING:
			return Types.VARCHAR;
		case PType.TYPECODE_DATE:
			return Types.DATE;
		case PType.TYPECODE_CHARARRAY:
			return Types.VARCHAR;
		case PType.TYPECODE_BYTEARRAY:
			return Types.VARBINARY;
		case PType.TYPECODE_SERIALIZED:
			return Types.VARBINARY;
		case PType.TYPECODE_BIGDECIMAL:
			return Types.DECIMAL;
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}

	public String getSqlType(PType pt, boolean usedInPK) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql Type name associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_OBJBOOLEAN:
			return "BIT";
		case PType.TYPECODE_CHAR:
		case PType.TYPECODE_OBJCHAR:
			return "CHAR";
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_OBJBYTE:
			return "TINYINT";
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_OBJSHORT:
			return "SMALLINT";
		case PType.TYPECODE_INT:
		case PType.TYPECODE_OBJINT:
			return "INTEGER";
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_OBJLONG:
			return "BIGINT";
		case PType.TYPECODE_FLOAT:
		case PType.TYPECODE_OBJFLOAT:
			return "REAL";
		case PType.TYPECODE_DOUBLE:
		case PType.TYPECODE_OBJDOUBLE:
			return "DOUBLE";
		case PType.TYPECODE_STRING:
			return "VARCHAR";
		case PType.TYPECODE_DATE:
			return "DATE";
		case PType.TYPECODE_CHARARRAY:
			return "VARCHAR";
		case PType.TYPECODE_BYTEARRAY:
			return "VARBINARY";
		case PType.TYPECODE_SERIALIZED:
			return "VARBINARY";
		case PType.TYPECODE_BIGDECIMAL:
			return "DECIMAL";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}

	public String getSqlGetterName(PType pt, String resultSet, int idx)
        throws PExceptionProtocol {
        if (pt == null) {
			throw new PExceptionProtocol("No Sql getter associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return resultSet + ".getBoolean(" + idx + ")";
		case PType.TYPECODE_CHAR:
			return resultSet + ".getString(" + idx + ").charAt(0)";
		case PType.TYPECODE_BYTE:
			return resultSet + ".getByte(" + idx + ")";
		case PType.TYPECODE_SHORT:
			return resultSet + ".getShort(" + idx + ")";
		case PType.TYPECODE_INT:
			return resultSet + ".getInt(" + idx + ")";
		case PType.TYPECODE_LONG:
			return resultSet + ".getLong(" + idx + ")";
		case PType.TYPECODE_FLOAT:
			return resultSet + ".getFloat(" + idx + ")";
		case PType.TYPECODE_DOUBLE:
			return resultSet + ".getDouble(" + idx + ")";
		case PType.TYPECODE_STRING:
			return resultSet + ".getString(" + idx + ")";
		case PType.TYPECODE_DATE:
			return resultSet + ".getDate(" + idx + ")";
		case PType.TYPECODE_BIGDECIMAL:
			return resultSet + ".getBigDecimal(" + idx + ")";
		case PType.TYPECODE_CHARARRAY:
			return resultSet + ".getBytes(" + idx + ")";
		case PType.TYPECODE_BYTEARRAY:
			return resultSet + ".getByteArray(" + idx + ")";
        case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJBOOLEAN:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_OBJFLOAT:
		case PType.TYPECODE_OBJDOUBLE:
            return "((" + pt.getJavaName() + ")"
                + resultSet + ".getObject(" + idx + "))";
        case PType.TYPECODE_SERIALIZED:
            return "(Serializable) new ObjectInputStream(" + resultSet
			+ ".getBlob(" + idx + ").getBinaryStream()).readObject()";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
    }

	public String getSqlSetterName(PType pt, String value, int idx) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql setter associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "setBoolean(" + idx + ", " + value + ")";
		case PType.TYPECODE_CHAR:
			return "setString(" + idx + ", " + "String.valueOf(" + value + "))";
		case PType.TYPECODE_BYTE:
			return "setByte(" + idx + ", " + value + ")";
		case PType.TYPECODE_SHORT:
			return "setShort(" + idx + ", " + value + ")";
		case PType.TYPECODE_INT:
			return "setInt(" + idx + ", " + value + ")";
		case PType.TYPECODE_LONG:
			return "setLong(" + idx + ", " + value + ")";
		case PType.TYPECODE_FLOAT:
			return "setFloat(" + idx + ", " + value + ")";
		case PType.TYPECODE_DOUBLE:
			return "setDouble(" + idx + ", " + value + ")";
		case PType.TYPECODE_STRING:
			return "setString(" + idx + ", " + value + ")";
		case PType.TYPECODE_DATE:
			return "setDate(" + idx + ", "
                + "(" + value + " instanceof java.sql.Date ? (java.sql.Date)"
				+ value + " : new java.sql.Date(" + value + ".getTime()))" + ")";
		case PType.TYPECODE_BIGDECIMAL:
			return "setBigDecimal(" + idx + ", " + value + ")";
		case PType.TYPECODE_CHARARRAY:
			return "setCharArray(" + idx + ", " + value + ")";
		case PType.TYPECODE_BYTEARRAY:
			return "setBytes(" + idx + ", " + value + ")";
		case PType.TYPECODE_SERIALIZED:
            return "setBlob(" + idx + ", new RdbBlob(" + value + "))";
		case PType.TYPECODE_OBJBOOLEAN:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_OBJFLOAT:
		case PType.TYPECODE_OBJDOUBLE:
			return "setObject(" + idx + ", " + value + ")";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}


	/**
	 * This calculates the size of a result set. If this feature is not supported
	 * by a database or its jdbc driver, this method returns -1.
	 * @ResultSet rs the result set which the size must be returned
	 */
	public int fetchResultSetSize(ResultSet rs) throws SQLException {
        try {
            return rs.getFetchSize();
        } catch (SQLException e) {
            return -1;
        }
	}

    public boolean getBoolean(ResultSet rs, int idx) throws SQLException {
        return rs.getBoolean(idx);
    }

    public Boolean getOboolean(ResultSet rs, int idx) throws SQLException {
        boolean res = rs.getBoolean(idx);
        return (rs.wasNull() ? null : new Boolean(res));
    }

    public char getChar(ResultSet rs, int idx) throws SQLException {
        return (char) rs.getInt(idx);
    }

    public Character getOchar(ResultSet rs, int idx) throws SQLException {
        char res = (char) rs.getInt(idx);
        return (rs.wasNull() ? null : new Character(res));
    }

    public byte getByte(ResultSet rs, int idx) throws SQLException {
        return rs.getByte(idx);
    }

    public Byte getObyte(ResultSet rs, int idx) throws SQLException {
        byte res = rs.getByte(idx);
        return (rs.wasNull() ? null : new Byte(res));
    }

    public short getShort(ResultSet rs, int idx) throws SQLException {
        return rs.getShort(idx);
    }

    public Short getOshort(ResultSet rs, int idx) throws SQLException {
        short res = rs.getShort(idx);
        return (rs.wasNull() ? null : new Short(res));
    }

    public int getInt(ResultSet rs, int idx) throws SQLException {
        return rs.getInt(idx);
    }

    public Integer getOint(ResultSet rs, int idx) throws SQLException {
        int res = rs.getInt(idx);
        return (rs.wasNull() ? null : new Integer(res));
    }

    public long getLong(ResultSet rs, int idx) throws SQLException {
        return rs.getLong(idx);
    }

    public Long getOlong(ResultSet rs, int idx) throws SQLException {
        long res = rs.getLong(idx);
        return (rs.wasNull() ? null : new Long(res));
    }

    public float getFloat(ResultSet rs, int idx) throws SQLException {
        return rs.getFloat(idx);
    }

    public Float getOfloat(ResultSet rs, int idx) throws SQLException {
        float res = rs.getFloat(idx);
        return (rs.wasNull() ? null : new Float(res));
    }

    public double getDouble(ResultSet rs, int idx) throws SQLException {
        return rs.getDouble(idx);
    }

    public Double getOdouble(ResultSet rs, int idx) throws SQLException {
        double res = rs.getDouble(idx);
        return (rs.wasNull() ? null : new Double(res));
    }

    public String getString(ResultSet rs, int idx) throws SQLException {
        return rs.getString(idx);
    }

    public Date getDate(ResultSet rs, int idx) throws SQLException {
        return rs.getDate(idx);
    }

    public char[] getCharArray(ResultSet rs, int idx) throws SQLException {
        return (char[]) rs.getObject(idx);
    }

    public byte[] getByteArray(ResultSet rs, int idx) throws SQLException {
        return rs.getBytes(idx);
    }

	public Object getObject(ResultSet rs, int idx) throws SQLException {
        return rs.getObject(idx);
    }

	public BigDecimal getBigDecimal(ResultSet rs, int idx) throws SQLException {
        return rs.getBigDecimal(idx);
    }
}
